/*
 * moat.cpp - Practicum, Spring 2009 week 1
 *
 * Anatole Paine
 *
 * Feb 12, 2009
 */

#include <cstddef>
#include <cmath>
#include <cstdlib>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

// Constants from the problem.
const int MAX_HOLES = 50000;
const int COORD_MAG = 20000;

bool DEBUG = false;

/*
 * A Point class to make our lives easier. Theoretically, we *could* use
 * std::pair, but that can raise strange templating and compiler issues. Hence,
 * we'll write a short, non-templated struct for this problem.
 */
struct Point
{
int x;
int y;

Point()
: x(0), y(0)
{
// Nothing to do...
}

Point(int xc, int yc)
: x(xc), y(yc)
{
// Nothing else to do here.
}

bool operator==(const Point& rhs)
{
return (x == rhs.x) && (y == rhs.y);
}
};

/*
 * Helper function: Distance between two Points, p1 and p2.
 * This is a convex hull problem, so we're going to be using this a lot.
 */
double dist(const Point& p1, const Point& p2)
{
return sqrt(pow((double)(p1.x - p2.x), 2) +
pow((double)(p1.y - p2.y), 2));
}

/*
 * Helper function: Binary predicate that checks whether the y coordinate
 * of the first Point is the lower of the two.
 */
bool lower(const Point& p1, const Point& p2)
{
return p1.y < p2.y;
}

/*
 * Helper function: Computes the cross product of three Points.
 */
int crossProd(const Point& p1, const Point& p2, const Point& p3)
{
return (p2.x - p1.x)*(p3.y - p1.y)
- (p3.x - p1.x)*(p2.y - p1.y);
}

/*
 * Function object: Since C++ will not support the lambda calculus until the
 * next ISO C++ standard comes out, and we need a callable binary predicate
 * that depends on a variable, we will have to use a function object. If only
 * we had Boost::lambda...
 * Since we're lazy, we're going to inline it.
 */
struct AngleFrom
{
Point _p;

AngleFrom(Point p)
: _p(p)
{
// Nothing (more) to do here.
}

bool operator()(const Point& p1, const Point& p2) const
{
// First we find the tangents of the angles that p1 and p2 make with
// _p, our target Point.
double tan1 = atan2(p1.y - _p.y, p1.x - _p.x);
double tan2 = atan2(p2.y - _p.y, p2.x - _p.x);

/*
 * Then we return true if tan1 is the smaller of the two - if they're
 * equal (implicit in the else case), then we return the lower of the
 * two distance-wise.
 */
if (tan1 < tan2)
return true;
else if (tan1 > tan2)
return false;
else
return dist(_p,p1) < dist(_p,p2);
}
};

/*
 * Main function: Graham's Scan, a convex hull algorithm.
 * This algorithm computes the convex hull for a given set of Points (specified
 * by the points parameter), and then computes the total length of the convex
 * hull afterwards. This should solve the entire problem fairly easily.
 */
double grahamScan(vector<Point>& points)
{
// Start with the lowest Point, which we first need to find:
Point pivot = *min_element(points.begin(), points.end(), lower);

/*
 * Now, we sort the vector of points with regard to their angle, using
 * atan2, which is in the cmath (a.k.a. math.h) library. Note that to do
 * this, we first have to instantiate a function object using our pivot.
 */
AngleFrom angleFromP = AngleFrom(pivot);
sort(points.begin(),points.end(), angleFromP);

// Just some error checking - we really want to be sure that our pivot is,
// in fact, the first Point in the sorted vector.
assert(pivot == points[0]);

// Now we can start the fun part. First we want a stack object to hold
// our Points, and a double to hold the accumulated distance.
vector<Point> stack;
double totalDistance = 0;

// Now, to start the algorithm, we push the pivot and the first Point in our
// newly sorted list onto the stack.
stack.push_back(points[0]);
stack.push_back(points[1]);

/*
 * Now for the main part of the algorithm. Basically, we loop over the
 * vector of points, adding points as long as we're making left turns, and
 * tallying up the distance traveled when we're done. This is guaranteed to
 * find the length of the smallest possible moat around Farmer Ran's farm,
 * as we know that the desired moat is the convex hull of the watering
 * holes. However, without this last part, this algorithm is simply Graham's
 * Scan.
 */
int i = 2;
const int N = points.size();
while (i < N)
{
int size = stack.size();
const Point a = stack.at(size - 1);
const Point b = stack.at(size - 2);

/*
 * Note the size <= 2 condition. This is important, as we never want the
 * stack to have fewer than two items - without three points, we don't
 * have a cross product.
 */
if (crossProd(b, a, points[i]) > 0 || size <= 2)
{
stack.push_back(points[i]);
++i;
}
else
{
stack.pop_back();
}
}

/*
 * Now that we have the convex hull, we want to add up the distances of the
 * line segments forming it.
 */
for (size_t i = 0; i < stack.size() - 1; ++i)
{
Point p1 = stack.at(i);
Point p2 = stack.at(i + 1);

if(DEBUG)
{
cerr << "Tallying, p1 is (" << p1.x << ',' << p1.y << ')' << endl;
cerr << "Tallying, p2 is (" << p2.x << ',' << p2.y << ')' << endl;
}

totalDistance += dist(p1,p2);
}

// We still have one line segment left, though - the one that closes the
// convex hull:
totalDistance += dist(stack.back(), stack.at(0));

// Now that we've computed the convex hull and the distance traveled along
// the fence, we're done!
return totalDistance;
}

/*
 * Function: The main driver for the program. This handles all of the ugly
 * input stuff, turns the input into useful data, and then runs the algorithm
 * on that data.
 */
int main() // We don't care about command-line stuff.
{

// Let's get some input.
string firstNum;
getline(cin, firstNum);
try
{
// From the input, we'll set our capacity and number of holes.
int holes = atol(firstNum.c_str());
vector<Point> points = vector<Point>();

// For each line after the first, add a new coordinate to the vector.
for(int i = 0; i < holes; ++i)
{
int x = 0;
int y = 0;
cin >> x >> y;

// Make sure the coordinates conform to the assignment.
if (abs(x) > COORD_MAG || abs(y) > COORD_MAG)
{
cerr << "Can't have points more than 40K units apart!" << endl;
exit(1);
}

Point coord = Point(x,y);
points.push_back(coord);
}

/*
 * We need to make sure that we avoid going outside the limits set by
 * the problem. Note that none of these are program-crashing problems,
 * but we'll quit anyway for safety.
 */
if (holes > MAX_HOLES)
{
cerr << "Too many watering holes!" << endl;
exit(1);
}

/*
 * Now, we can finally do some computation. Note that the problem
 * statement says that we should only return the first two decimal
 * places, so we'll use the std::fixed and std::setprecision
 * methods to change cout output so that it only outputs floating -
 * point numbers with precision to the hundreds place.
 */
cout << setprecision(2) << fixed << grahamScan(points) << endl;

}
catch(const exception& e)
{
cerr << "Something went wrong:" << e.what() << endl;
return(1);
}

return 0;
}
