/*
 * mtime.X - Practicum, Spring 2009 week 1
 *
 * Anatole Paine and Emma Carlson
 *
 * May 16, 2009
 */

#include <cstddef>
#include <cmath>
#include <cstdlib>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

// Constants from the problem.
const int MAX_N = 1000;
const int MAX_T = 1000;
const int MAX_S = 1000000;

// A struct to help deal with data containment.
struct Job
{
    int t;
    int s;
    int lazy;
};

/*
 * Helper function: Strict weak ordering property for use in std::sort.
 * It allows us to sort the list such that the jobs are in reverse order of the
 * latest they can be completed by.
 */
bool compareEnds(const Job& lhs, const Job& rhs)
{
    if (lhs.s > rhs.s)
        return true;
    else if (rhs.s == lhs.s)
        return lhs.lazy > rhs.lazy;
    else
        return false;
}

/*
 * Main function: Greedy algorithm. It iterates through the list of jobs,
 * pushing them back until they all fit into FR's schedule.
 */
int manageTime(vector<Job>& jobs)
{
    /* 
     * This problem relies on a partial ordering property, so we first need
     * to sort the vector of jobs by the latest they can be completed by.
     */
    sort(jobs.begin(), jobs.end(), compareEnds);

    /*
     * The main part here: Essentially a greedy algorithm. We start the search
     * at the last job, since we can always complete that (hopefully). For each
     * job, we look at the job right before it (temporally, that is - in the
     * vector, the job after it), and if that job overlaps, we simply push it
     * back until it doesn't overlap anymore.
     */
    for (size_t i = 0; i < jobs.size() - 1; ++i)
    {
        /* Debug code.
        cerr << "Currently looking at element " << i << ": {" << jobs.at(i).t;
        cerr << ", " << jobs.at(i).s << ", " << jobs.at(i).lazy << "}" << endl;
        */

        // Store the latest time we can start the current job.
        int lazy = jobs.at(i).lazy;

        // If the jobs overlap, push the previous job back.
        if (jobs.at(i).lazy <= jobs.at(i + 1).s)
        {
            jobs.at(i + 1).s = lazy;
            jobs.at(i + 1).lazy = lazy - jobs.at(i + 1).t;
        }
    }

    /* 
     * Because of the way this algorithm works, if it's impossible for FR to 
     * complete all of his jobs on time, we will get a negative number. The
     * specification for the problem, however, says that this number must be
     * exactly -1, so we'll set it to that.
     */
    int result = jobs.at(jobs.size() - 1).lazy;
    if (result < 0)
        result = -1;
    
    return result;
}

bool DEBUG = false;
/*
 * Function: The main driver for the program. This handles all of the ugly
 * input stuff, turns the input into useful data, and then runs the algorithm
 * on that data.
 */
int main() // We don't care about command-line stuff.
{
    
    // Let's get some input.
    string firstNum;
    getline(cin, firstNum);
    try
    {
        // From the input, we'll get the number of jobs FR must do.
        int N = atol(firstNum.c_str());

        /*
         * We need to make sure that we avoid going outside the limits set by
         * the problem. Note that none of these are program-crashing problems,
         * but we'll quit anyway for safety.
         */
        if (N > MAX_N)
        {
            cerr << "Too many jobs!" << endl;
            exit(1);
        }

        // We need a data structure to hold our jobs.
        vector<Job> jobs;

        // For each line after the first, add a new job to the vector.
        for(int i = 0; i < N; ++i)
        {
            int t = -1;
            int s = -1;
            cin >> t >> s;

            // Make sure the times specified conform to the assignment.
            if (s > MAX_S || t > MAX_T)
            {
                cerr << "Job length or end time too long!" << endl;
                exit(1);
            }

            Job current = {t, s, s - t};
            jobs.push_back(current);
        }

        /* 
         * Now, we can finally do some computation. Since our manageTime
         * function returns an int, we'll just print it.
         */
        cout << manageTime(jobs) << endl;
    }
    catch(const exception& e)
    {
        cerr << "Something went wrong:" << e.what() << endl;
        return(1);
    }

    return 0;
}
